<?php

/**
 *
 * This class is responsible to send SMS using API.
 */
class Woo_Usn_SMS
{
    /**This variable contains all the data related to the SMS API object.
     *
     * @var object
     */
    private  $sms_api ;
    /**
     * Class constructor.
     */
    public function __construct()
    {
        global  $usn_utility ;
        $this->sms_api = json_decode( $usn_utility::get_api_credentials() );
    }
    
    /**
     * Send SMS using Twilio Rest API.
     *
     * @return mixed
     */
    private function send_sms_with_rest(
        $id,
        $token,
        $to,
        $body,
        $from = false
    )
    {
        $headers = array(
            'Authorization' => 'Basic ' . base64_encode( $id . ':' . $token ),
        );
        
        if ( $from ) {
            $url = "https://api.twilio.com/2010-04-01/Accounts/{$id}/Messages.json";
            $data = array(
                'From' => $from,
                'To'   => $to,
                'Body' => $body,
            );
        } else {
            $url = 'https://rest-api.telesign.com/v1/messaging';
            $data = array(
                'phone_number' => $to,
                'message'      => $body,
                'message_type' => 'ARN',
            );
        }
        
        $result = wp_remote_post( $url, array(
            'body'    => $data,
            'headers' => $headers,
        ) );
        $body_result = wp_remote_retrieve_body( $result );
        $decoded = json_decode( $body_result );
        
        if ( $from ) {
            return $decoded->status;
        } else {
            
            if ( null !== $decoded->status->code ) {
                return $decoded->status->code;
            } else {
                return 400;
            }
        
        }
    
    }
    
    /**
     * This functions send SMS to phone numbers using the SMS API defined.
     *
     * @param string $phone_number Customer phone number.
     * @param string $message_to_send Message to send to customer.
     * @return bool|string
     */
    public final function send_sms( $phone_number, $message_to_send )
    {
        
        if ( 'Twilio' === $this->sms_api->api_used ) {
            $from_number = get_option( 'woo_usn_twilio_phone_number' );
            if ( !strpos( '+', $phone_number ) ) {
                $phone_number = '+' . $phone_number;
            }
            return $this->send_sms_with_rest(
                $this->sms_api->first_api_key,
                $this->sms_api->second_api_key,
                $phone_number,
                $message_to_send,
                $from_number
            );
        } elseif ( 'Telesign' === $this->sms_api->api_used ) {
            return $this->send_sms_with_rest(
                $this->sms_api->first_api_key,
                $this->sms_api->second_api_key,
                $phone_number,
                $message_to_send,
                false
            );
        } elseif ( 'Kivalo' === $this->sms_api->api_used ) {
            $from_number = $this->sms_api->first_api_key;
            $api_key = $this->sms_api->second_api_key;
            $url = "http://sms.kivalosolutions.com/sms/api?action=send-sms&api_key={$api_key}&to={$phone_number}&from={$from_number}&sms={$message_to_send}";
            $response = wp_remote_get( $url, array(
                'timeout' => 30,
            ) );
            $response_body = wp_remote_retrieve_body( $response );
            $decoded_body = json_decode( $response_body );
            return $decoded_body->code;
        } elseif ( 'Waapi' === $this->sms_api->api_used ) {
            $client_id = $this->sms_api->first_api_key;
            $instance_id = $this->sms_api->second_api_key;
            $domain_url = get_option( 'woo_usn_waapi_domain_url' );
            $url = "{$domain_url}//api/send.php?client_id={$client_id}&instance={$instance_id}&type=text&number={$phone_number}&message={$message_to_send}";
            $response = wp_remote_get( $url, array(
                'timeout' => 30,
            ) );
            $response_body = wp_remote_retrieve_body( $response );
            $decoded_body = json_decode( $response_body );
            
            if ( $decoded_body->status ) {
                return 200;
            } else {
                return 400;
            }
        
        }
        
        /**
         * you have the ability to send sms using a external sms api if you don't want to use the previous ones.
         */
        return apply_filters(
            'woo_usn_send_sms_to_customer',
            $this->sms_api->api_used,
            $phone_number,
            $message_to_send
        );
    }
    
    /**
     * This method allows to send SMS based on the data related at a WC order.
     *
     * @param  int $order WooCommerce Order ID.
     * @return void
     */
    public function send_api_messages( $order )
    {
        global  $usn_utility ;
        $log = new WC_Logger();
        $_order = new WC_Order( $order );
        $country = $_order->get_billing_country();
        $_phone_number = $usn_utility::get_right_phone_numbers( $country, $_order->get_billing_phone() );
        if ( !isset( $_phone_number ) && !isset( $country ) ) {
            return;
        }
        $country_indicator = $usn_utility::get_country_town_code( $country );
        $phone_number = $country_indicator . $usn_utility::get_right_phone_numbers( $country_indicator, $_phone_number );
        $options = get_option( 'woo_usn_options' );
        if ( isset( $options['woo_usn_message_after_customer_purchase'] ) ) {
            $customer_message = $usn_utility::decode_message_to_send( $order, $options['woo_usn_defaults_messages'] );
        }
        
        if ( isset( $options['woo_usn_sms_to_admin'] ) ) {
            $admin_can_receive_messages = esc_attr( $options['woo_usn_sms_to_admin'] );
            $admin_numbers = esc_attr( $options['woo_usn_admin_numbers'] );
            $admin_message = $usn_utility::decode_message_to_send( $order, $options['woo_usn_admin_messages_template'] );
        }
        
        $success_message_return = __( 'SMS successfully sent', 'ultimate-sms-notifications' );
        $failed_message_return = __( 'SMS failed to send , please again later .', 'ultimate-sms-notifications' );
        // send SMS to customer.
        try {
            
            if ( isset( $customer_message ) && !is_admin() ) {
                $return = $this->send_sms( $phone_number, $customer_message );
                
                if ( 'queued' == $return || preg_match( '/^2([0-9]{1})([0-9]{1})$/', $return ) ) {
                    $return = $success_message_return;
                } else {
                    $return = $failed_message_return;
                }
                
                $orders_messages = '<br/><strong>' . __( 'Phone Numbers : ', 'ultimate-sms-notifications' ) . '</strong>' . $phone_number . '<br/><strong>' . __( 'Messages : ', 'ultimate-sms-notifications' ) . '</strong>' . $customer_message . '<br/><strong>' . __( 'Delivery Messages : ', 'ultimate-sms-notifications' ) . '</strong>' . $return . '<br/>' . 'Sent from <strong>Ultimate SMS Notifications for WooCommerce</strong>';
                $_order->add_order_note( $orders_messages );
            }
        
        } catch ( Exception $errors ) {
            $log->error( __( 'An error has occured, while sending SMS to customers : ', 'ultimate-sms-notifications' ) . $errors, array(
                'source' => 'ultimate-sms-notifications',
            ) );
            $return = $failed_message_return;
            $orders_messages = '<br/><strong>' . __( 'Phone Numbers : ', 'ultimate-sms-notifications' ) . '</strong>' . $phone_number . '<br/><strong>' . __( 'Messages : ', 'ultimate-sms-notifications' ) . '</strong>' . $customer_message . '<br/><strong>' . __( 'Delivery Messages : ', 'ultimate-sms-notifications' ) . '</strong>' . $return . '<br/>' . 'Sent from <strong>Ultimate SMS Notifications for WooCommerce</strong>';
            $_order->add_order_note( $orders_messages );
        }
        // send SMS to shop manager.
        try {
            
            if ( isset( $admin_can_receive_messages ) && !is_admin() ) {
                $return = $this->send_sms( $admin_numbers, $admin_message );
                
                if ( 'queued' == $return || preg_match( '/^2([0-9]{1})([0-9]{1})$/', $return ) ) {
                    $return = $success_message_return;
                } else {
                    $return = $failed_message_return;
                }
                
                $orders_messages = '<br/><strong>' . __( 'Phone Numbers : ', 'ultimate-sms-notifications' ) . '</strong>' . $admin_numbers . '<br/><strong>' . __( 'Messages : ', 'ultimate-sms-notifications' ) . '</strong>' . $admin_message . '<br/><strong>' . __( 'Delivery Messages : ', 'ultimate-sms-notifications' ) . '</strong>' . $return . '<br/>' . 'Sent from <strong>Ultimate SMS Notifications for WooCommerce</strong>';
                $_order->add_order_note( $orders_messages );
            }
        
        } catch ( Exception $errors ) {
            $log->error( __( 'An error has occured, while sending SMS to shop-manager : ', 'ultimate-sms-notifications' ) . $errors, array(
                'source' => 'ultimate-sms-notifications',
            ) );
            $return = $failed_message_return;
            $orders_messages = '<br/><strong>' . __( 'Phone Numbers : ', 'ultimate-sms-notifications' ) . '</strong>' . $phone_number . '<br/><strong>' . __( 'Messages : ', 'ultimate-sms-notifications' ) . '</strong>' . $customer_message . '<br/><strong>' . __( 'Delivery Messages : ', 'ultimate-sms-notifications' ) . '</strong>' . $return . '<br/>' . 'Sent from <strong>Ultimate SMS Notifications for WooCommerce</strong>';
            $_order->add_order_note( $orders_messages );
        }
    }

}